// apachego -- a launcher for apache
//
// Copyright (C) 2007
// Wolfgang Formann
//
// this file is part of the programm patchvista.  This program is
// free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your
// option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include "AgWindow.h"

#include "AboutProgramm.h"
#include "FillCache.h"

#include <assert.h>
#include <errno.h>
#include <iostream>
#include <limits.h>
#include <sys/stat.h>
#ifdef Q_OS_WIN
#	include <malloc.h>
#	include <windows.h>
#	include <io.h>
#	include <tchar.h>
#   include <winerror.h>
#   include <winsock.h>
#else
#   include <netdb.h>
#   include <sys/socket.h>
#   include <netinet/in.h>
#   include <sys/signal.h>
#endif

#include <Qt/qapplication.h>
#include <Qt/qdesktopwidget.h>
#include <Qt/qstackedlayout.h>
#include <Qt/qlabel.h>
#include <Qt/qpushbutton.h>
#include <Qt/qcheckbox.h>
#include <Qt/qdir.h>
#include <Qt/qevent.h>
#include <Qt/qtimer.h>

AgWindow* AgWindow::instance = 0;

AgWindow::AgWindow()
	: QDialog(),
      _apache(0),
      _apacheStarted(false),
      _browser(0)
{
    // size
//    QRect rect = QApplication::desktop()->screenGeometry();
//    int width = rect.width();
//    if(width > 1280)
//        width = 1280;
//    int height = rect.height();
//    if(height > 1024)
//        height = 1024;
//	resize(width / 3, height / 3);

    setWindowTitle(tr("Window Title", ""));
    setWindowFlags(Qt::Window);

    _layers = new QStackedLayout(this);
    _layers->addWidget(page1());
    _layers->addWidget(page2());
    _layers->addWidget(page3());
}

QWidget* AgWindow::page1()
{
    QWidget *thePage = new QWidget();
    QVBoxLayout *layoutThePage = new QVBoxLayout();
    thePage->setLayout(layoutThePage);

    QHBoxLayout *layoutTopPage = new QHBoxLayout();
	QLabel *logoGraphic = new QLabel();
    if(access("logos/startlogo.png", R_OK) == 0)
        logoGraphic->setPixmap(QPixmap("logos/startlogo.png"));
    else
        logoGraphic->setPixmap(QPixmap(":/images/logo"));
    layoutTopPage->addWidget(logoGraphic, 0);

    // the individual part of page 1
    QLabel *headerText = 0;
    switch(checkInstallation()) {
    case eMISS_FILES:
        headerText = new QLabel(tr("Es fehlen wichtige Dateien.", ""));
        break;

    case eMISS_TEMP:
        headerText = new QLabel(tr("Die Umgebungsvariable %TEMP% ist nicht gesetzt.", ""));
        break;

    case eMISS_ACCESS: {
            QString text = tr("Sie haben kein Schreibrecht auf das temporre Verzeichnis \"", "");
            text += getenv("TEMP");
            headerText = new QLabel(text + tr("\".", ""));
        }
        break;

    default:
        layoutTopPage->addWidget(subPage1());
        FillCache *reader = new FillCache();
        reader->start();
    }

    if(headerText != 0) {
        QFont f(headerText->font());
        f.setBold(true);
        if(f.pixelSize() == -1) {
            if(f.pointSize() == -1)
                assert(0);
            f.setPointSize(f.pointSize() + 2);
        }
        else {
            f.setPixelSize(f.pixelSize() + 2);
        }
        headerText->setFont(f);

        QVBoxLayout *layoutTopRightPage = new QVBoxLayout();
        QWidget *theInnerPage = new QWidget();
        theInnerPage->setLayout(layoutTopRightPage);
 
        QLabel *header1Text = new QLabel(tr("Programm Title", ""));
        QFont f2(header1Text->font());
        f2.setBold(true);
        if(f2.pixelSize() == -1) {
            if(f2.pointSize() == -1)
                assert(0);
            f2.setPointSize(f2.pointSize() + 10);
        }
        else {
            f2.setPixelSize(f2.pixelSize() + 10);
        }
        header1Text->setFont(f2);
        layoutTopRightPage->addSpacing(18);
        layoutTopRightPage->addWidget(header1Text, 0, Qt::AlignCenter);

        headerText->setWordWrap(true);
        layoutTopRightPage->addStretch(1);
        layoutTopRightPage->addWidget(headerText);
        layoutTopRightPage->addStretch(2);
        layoutTopPage->addWidget(theInnerPage, 1);
    }
    // end of individual part of page 1

    layoutThePage->addLayout(layoutTopPage);

    QHBoxLayout *layoutButtons1 = new QHBoxLayout();
    QPushButton *cancel1 = new QPushButton(tr("Been&den", ""), this);
    connect(cancel1, SIGNAL(clicked()), this, SLOT(doCancel()));
   	layoutButtons1->addWidget(cancel1);

    QPushButton *about1 = new QPushButton(tr("ber", ""), this);
    connect(about1, SIGNAL(clicked()), this, SLOT(doAbout()));
   	layoutButtons1->addWidget(about1);

   	layoutButtons1->addStretch(1);
    layoutThePage->addLayout(layoutButtons1, 0);

    return thePage;
}

QWidget* AgWindow::page2()
{
    QWidget *thePage = new QWidget();
    QVBoxLayout *layoutThePage = new QVBoxLayout();
    thePage->setLayout(layoutThePage);

    QHBoxLayout *layoutTopPage = new QHBoxLayout();
	QLabel *logoGraphic = new QLabel();
    if(access("logos/startlogo.png", R_OK) == 0)
        logoGraphic->setPixmap(QPixmap("logos/startlogo.png"));
    else
        logoGraphic->setPixmap(QPixmap(":/images/logo"));
    layoutTopPage->addWidget(logoGraphic, 0);

    // the individual part of page 1
    _page2Header = new QLabel("Platzhalter fr Fehlermeldungen");
    QFont f(_page2Header->font());
    f.setBold(true);
    if(f.pixelSize() == -1) {
        if(f.pointSize() == -1)
            assert(0);
        f.setPointSize(f.pointSize() + 2);
    }
    else {
        f.setPixelSize(f.pixelSize() + 2);
    }
    _page2Header->setFont(f);

    QVBoxLayout *layoutTopRightPage = new QVBoxLayout();
    QWidget *theInnerPage = new QWidget();
    theInnerPage->setLayout(layoutTopRightPage);

    QLabel *header1Text = new QLabel(tr("Programm Title", ""));
    QFont f2(header1Text->font());
    f2.setBold(true);
    if(f2.pixelSize() == -1) {
        if(f2.pointSize() == -1)
            assert(0);
        f2.setPointSize(f2.pointSize() + 10);
    }
    else {
        f2.setPixelSize(f2.pixelSize() + 10);
    }
    header1Text->setFont(f2);
    layoutTopRightPage->addSpacing(18);
    layoutTopRightPage->addWidget(header1Text, 0, Qt::AlignCenter);

    _page2Header->setWordWrap(true);
    layoutTopRightPage->addStretch(1);
    layoutTopRightPage->addWidget(_page2Header);
    layoutTopRightPage->addStretch(2);
    layoutTopPage->addWidget(theInnerPage, 1);
    // end of individual part of page 1

    layoutThePage->addLayout(layoutTopPage);

    QHBoxLayout *layoutButtons1 = new QHBoxLayout();
    QPushButton *cancel1 = new QPushButton(tr("Been&den", ""), this);
    connect(cancel1, SIGNAL(clicked()), this, SLOT(doCancel()));
   	layoutButtons1->addWidget(cancel1);

    QPushButton *about1 = new QPushButton(tr("ber", ""), this);
    connect(about1, SIGNAL(clicked()), this, SLOT(doAbout()));
   	layoutButtons1->addWidget(about1);

   	layoutButtons1->addStretch(1);
    layoutThePage->addLayout(layoutButtons1, 0);

    return thePage;
}

QWidget* AgWindow::page3()
{
    QWidget *thePage = new QWidget();
    QVBoxLayout *layoutThePage = new QVBoxLayout();
    thePage->setLayout(layoutThePage);

    QHBoxLayout *layoutTopPage = new QHBoxLayout();
	QLabel *logoGraphic = new QLabel();
    if(access("logos/startlogo.png", R_OK) == 0)
        logoGraphic->setPixmap(QPixmap("logos/startlogo.png"));
    else
        logoGraphic->setPixmap(QPixmap(":/images/logo"));
    layoutTopPage->addWidget(logoGraphic, 0);

    // the individual part of page 1
    QLabel *page2Header = new QLabel(tr("Beenden Sie das Programm erst, wenn Sie fertig geguckt haben.", ""));
    QFont f(page2Header->font());
    f.setBold(true);
    if(f.pixelSize() == -1) {
        if(f.pointSize() == -1)
            assert(0);
        f.setPointSize(f.pointSize() + 2);
    }
    else {
        f.setPixelSize(f.pixelSize() + 2);
    }
    page2Header->setFont(f);

    QVBoxLayout *layoutTopRightPage = new QVBoxLayout();
    QWidget *theInnerPage = new QWidget();
    theInnerPage->setLayout(layoutTopRightPage);

    QLabel *header1Text = new QLabel(tr("Programm Title", ""));
    QFont f2(header1Text->font());
    f2.setBold(true);
    if(f2.pixelSize() == -1) {
        if(f2.pointSize() == -1)
            assert(0);
        f2.setPointSize(f2.pointSize() + 10);
    }
    else {
        f2.setPixelSize(f2.pixelSize() + 10);
    }
    header1Text->setFont(f2);
    layoutTopRightPage->addSpacing(18);
    layoutTopRightPage->addWidget(header1Text, 0, Qt::AlignCenter);

    page2Header->setWordWrap(true);
    layoutTopRightPage->addStretch(1);
    layoutTopRightPage->addWidget(page2Header);
    layoutTopRightPage->addStretch(2);
    layoutTopPage->addWidget(theInnerPage, 1);
    // end of individual part of page 1

    layoutThePage->addLayout(layoutTopPage);

    QHBoxLayout *layoutButtons1 = new QHBoxLayout();
    _cancel3 = new QPushButton(tr("Been&den", ""), this);
    connect(_cancel3, SIGNAL(clicked()), this, SLOT(doCancel()));
    _cancel3->setEnabled(false);
   	layoutButtons1->addWidget(_cancel3);

    _timer = new QTimer(this);
    _timer->setInterval(5000);
    _timer->setSingleShot(true);
    connect(_timer, SIGNAL(timeout()), this, SLOT(enableExit()));

    QPushButton *about1 = new QPushButton(tr("ber", ""), this);
    connect(about1, SIGNAL(clicked()), this, SLOT(doAbout()));
   	layoutButtons1->addWidget(about1);

   	layoutButtons1->addStretch(1);
    layoutThePage->addLayout(layoutButtons1, 0);

    return thePage;
}

AgWindow::~AgWindow()
{
}

QWidget* AgWindow::subPage1()
{
    QWidget *thePage = new QWidget();
    QVBoxLayout *layoutTopRightPage = new QVBoxLayout();
    thePage->setLayout(layoutTopRightPage);

    QLabel *header1Text = new QLabel(tr("Programm Title", ""));
    QFont f2(header1Text->font());
    f2.setBold(true);
    if(f2.pixelSize() == -1) {
        if(f2.pointSize() == -1)
            assert(0);
        f2.setPointSize(f2.pointSize() + 10);
    }
    else {
        f2.setPixelSize(f2.pixelSize() + 10);
    }
    header1Text->setFont(f2);
    layoutTopRightPage->addSpacing(18);
    layoutTopRightPage->addWidget(header1Text, 0, Qt::AlignCenter);
    layoutTopRightPage->addStretch(1);

//    _usePortableFirefox = new QCheckBox(tr("Mitgelieferten Browser verwenden", ""));
//    if(access(tr("startfirefox", "").toAscii(), R_OK) == 0) {
//        _usePortableFirefox->setCheckState(Qt::Checked);
//    }
//    else {
//        _usePortableFirefox->setCheckState(Qt::Unchecked);
//        _usePortableFirefox->setEnabled(false);
//    }
//    layoutTopRightPage->addWidget(_usePortableFirefox);

    _start = new QPushButton(tr("Mitgelieferten Browser starten", ""), this);
    connect(_start, SIGNAL(clicked()), this, SLOT(doStart()));
   	layoutTopRightPage->addWidget(_start);

    return thePage;
}

AgWindow::InstallationStatus AgWindow::checkInstallation()
{
    if(access("apache22", R_OK) != 0)
        return eMISS_FILES;
    if(access("php5", R_OK) != 0)
        return eMISS_FILES;
    char *temp = getenv("TEMP");
    if(temp == 0)
#ifdef Q_OS_WIN
        return eMISS_TEMP;
#else
        temp = "/tmp";
#endif
    if(access(temp, W_OK) != 0)
        return eMISS_ACCESS;
    return eMISS_ALLOKAY;
}

AgWindow* AgWindow::getInstance()
{
	if(instance == 0)
		instance = new AgWindow();
	assert(instance != 0);
	return instance;
}

#ifdef Q_OS_WIN
BOOL CALLBACK killCallback(HWND handle, LPARAM param)
{
    DWORD dwID;
    GetWindowThreadProcessId(handle, &dwID);
    
    if(dwID == (DWORD)param)
        PostMessage(handle, WM_CLOSE, 0, 0);
    
    return TRUE;
}
#endif

void AgWindow::closeEvent(QCloseEvent *event)
{
    if(_apache) {
        _apache->disconnect();
        _apache->kill();
        if(!_apache->waitForFinished(6000)) {
#ifndef NDEBUG
            std::cerr << "failed to kill apache" << std::endl;
#endif
        }
        delete _apache;
        _apache = 0;
    }

    if(_apacheStarted) {
        QString errorLog(_tmpDir);
#ifdef Q_OS_WIN
        errorLog += '\\';
#else
        errorLog += '/';
#endif
        errorLog += "error.log";
        FILE *errLogFile = fopen(errorLog.toAscii(), "r");
        if(errLogFile != NULL) {
#ifndef NDEBUG
            std::cerr << "reading file" << std::endl;
#endif
            char databuffer[1024];
            int readChars;
            QString errLogContent;
            while((readChars = fread(databuffer, 1, sizeof(databuffer) - 1, errLogFile)) > 0) {
                databuffer[readChars] = 0;
                errLogContent += databuffer;
            }
            fclose(errLogFile);
            
#ifndef NDEBUG
            std::cerr << "Logfile:" << errLogContent.toStdString() << std::endl;
#endif

            int pos = errLogContent.indexOf("Child ");
#ifndef NDEBUG
            std::cerr << "Child at " << pos << std::endl;
#endif
            if(pos > 0) {
                int pid = 0;
                pos += 6;
                while(errLogContent.at(pos) >= '0' && errLogContent.at(pos) <= '9') {
                    pid = pid * 10 + errLogContent.at(pos).digitValue();
                    ++ pos;
                }
#ifndef NDEBUG
                std::cerr << "pid: " << pid << std::endl;
#endif
#ifdef Q_OS_WIN
                HANDLE processHandle;
                if((processHandle = OpenProcess(SYNCHRONIZE | PROCESS_TERMINATE, FALSE, pid)) != 0) {
                    EnumWindows((WNDENUMPROC)killCallback, (LPARAM)pid);

                    if(WaitForSingleObject(processHandle, 1000) != WAIT_OBJECT_0) {
                        if(TerminateProcess(processHandle, 0) == 0) {
                            CloseHandle(processHandle);
                            showError(eKILL_Apache);
                            event->ignore();
                            return;
                        }
                    }
                    CloseHandle(processHandle);
                }
                else {
                    showError(eKILL_Apache);
                    event->ignore();
                    return;
                }
#else
                kill(pid, SIGTERM);
#endif
            }
        }
        _apacheStarted = false;
    }
    if(!_tmpDir.isEmpty()) {
        cleanUp(_tmpDir, true);

        QDir tmpDir(getenv("TEMP") ? getenv("TEMP") : "C:\\");
        QFileInfoList content = tmpDir.entryInfoList();
        for(int i = 0; i < content.size(); ++ i) {
            if(content.at(i).baseName().isEmpty())
                continue;
            if(!content.at(i).isDir() && content.at(i).baseName().left(5) == "sess_") {
#ifndef NDEBUG
                std::cerr << "May delete:" << content.at(i).baseName().toStdString() << std::endl;
                std::cerr << "length():" << content.at(i).baseName().length() << std::endl;
                if(content.at(i).baseName().length() > 5)
                    std::cerr << "pos 5:" << content.at(i).baseName().at(5).toAscii() << std::endl;
#endif
                if(content.at(i).baseName().length() > 5 &&
                   (content.at(i).baseName().at(5) >= '0' && content.at(i).baseName().at(5) <= '9' ||
                    content.at(i).baseName().at(5) >= 'a' && content.at(i).baseName().at(5) <= 'f' ||
                    content.at(i).baseName().at(5) >= 'A' && content.at(i).baseName().at(5) <= 'F'))
                    unlink(content.at(i).absoluteFilePath().toAscii());
            }
        }
    }
    event->accept();
}

void AgWindow::doCancel()
{
    QDialog::close();
}

void AgWindow::enableExit()
{
    _cancel3->setEnabled(true);
    showMinimized();
#ifndef NDEBUG
    std::cerr << "Tick: " << std::endl;
#endif
//    delete _timer;
//    _timer = 0;
}

void AgWindow::showError(StartErrors error)
{
    switch(error) {
    case eSTART_GetPWD:
        // internal error, cannot get current working directory
        _page2Header->setText(tr("Kann das aktuelle Directory nicht feststellen", ""));
        break;

    case eSTART_EnvTemp:
        // no TEMP environment variable, cannot write to root in C:
        _page2Header->setText(tr("Es fehlt die Environmentvariable %TEMP%.", ""));
        break;

    case eSTART_EmptyPhpIni:
        // setup error, empty php.ini template file
        _page2Header->setText(tr("Die Vorlagedatei php.nin ist leer.", ""));
        break;

    case eSTART_MissPhpIni:
        // setup error, missing php.ini template file
        _page2Header->setText(tr("Es fehlt die Vorlagedatei php.nin.", ""));
        break;

    case eSTART_Apache:
        // could not start apache
        _page2Header->setText(tr("Der Webserver konnte nicht gestartet werden.", ""));
        break;

    case eKILL_Apache:
        // could not kill apache
        _page2Header->setText(tr("Der Webserver konnte nicht beendet werden. Bitte ffnen Sie den Taskmanager, wechseln Sie dort in den Reiter \"Prozesse\" und beenden Sie dort alle Prozesse mit dem Namen \"httpd.exe\"", ""));
        break;

    case eDIED_Apache:
        // apache died somehow
        _page2Header->setText(tr("Der Webserver hat sich unerwartet beendet.", ""));
        break;

    case eSTART_FireFox:
        // could not start firefox
        _page2Header->setText(tr("Der mitgelieferte Browser konnte nicht gestartet werden.", ""));
        break;
    }
    _layers->setCurrentIndex(1);
}

void AgWindow::doStart()
{
    if(_apacheStarted == false) {
        QString startString = tr("startapache", "");
        char buffer[PATH_MAX];
        if(getcwd(buffer, sizeof(buffer)) == 0) {
            *buffer = 0;
            showError(eSTART_GetPWD);
            return;
        }
    
        QString myPwd(buffer);
        if(myPwd.at(myPwd.length() - 1) != '\\' && myPwd.at(myPwd.length() - 1) != '/')
#ifdef Q_OS_WIN
            myPwd += '\\';
#else
            myPwd += '/';
#endif

#ifndef NDEBUG
        std::cerr << "Start-string: " << startString.toStdString() << std::endl;
#endif
        for(int pos = 0; (pos = startString.indexOf("E:\\portable_apache\\", pos)) > 0; ) {
            startString.replace(pos, 19, myPwd);
            pos += myPwd.length();
        }

        _tmpDir = getTempDir();
        if(_tmpDir.at(_tmpDir.length() - 1) != '\\' && _tmpDir.at(_tmpDir.length() - 1) != '/')
#ifdef Q_OS_WIN
            _tmpDir += '\\';
#else
            _tmpDir += '/';
#endif
        _tmpDir += tr("mfs-wfo-apache-cb0815", "");
        struct stat statBuf;
        if(stat(_tmpDir.toAscii(), &statBuf) == 0) {
            if(S_ISDIR(statBuf.st_mode)) {
                cleanUp(_tmpDir, false);
            }
            else {
                unlink(_tmpDir.toAscii());
#ifdef Q_OS_WIN
                mkdir(_tmpDir.toAscii());
#else
                mkdir(_tmpDir.toAscii(), 0700);
#endif
            }
        }
        else {
#ifdef Q_OS_WIN
            mkdir(_tmpDir.toAscii());
#else
            mkdir(_tmpDir.toAscii(), 0700);
#endif
        }

        for(int pos = 0; (pos = startString.indexOf("%TEMP%", pos)) > 0; ) {
            startString.replace(pos, 6, _tmpDir);
            pos += _tmpDir.length();
        }

#ifndef NDEBUG
        std::cerr << "Start-string: " << startString.toStdString() << std::endl;
#endif

        FILE *iniFile = fopen(tr("php5-ini-template", "").toAscii(), "r");
        if(iniFile != NULL) {
            char databuffer[1024];
            int readChars;
            QString iniContent;
            while((readChars = fread(databuffer, 1, sizeof(databuffer) - 1, iniFile)) > 0) {
                databuffer[readChars] = 0;
                iniContent += databuffer;
            }
            fclose(iniFile);
        
            if(iniContent.isEmpty()) {
                showError(eSTART_EmptyPhpIni);
                return;
            }
    
            for(int pos = 0; (pos = iniContent.indexOf("E:\\portable_apache\\", pos)) > 0; ) {
                iniContent.replace(pos, 19, myPwd);
                pos += myPwd.length();
            }
    
            QString newIniFile(_tmpDir);
#ifdef Q_OS_WIN
            newIniFile += "\\php.ini";
#else
            newIniFile += "/php.ini";
#endif
#ifndef NDEBUG
            std::cerr << "ini-file: " << newIniFile.toStdString() << std::endl;
#endif
            if((iniFile = fopen(newIniFile.toAscii(), "w")) == 0) {
                showError(eSTART_EnvTemp);
                return;
            }
            fwrite(iniContent.toAscii(), 1, iniContent.length(), iniFile);
            fclose(iniFile);
        }
        else {
            showError(eSTART_MissPhpIni);
            return;
        }
        _apache = new QProcess(this);
        _apache->setProcessChannelMode(QProcess::ForwardedChannels);
        _apache->start(startString);
        if(!_apache->waitForStarted(6000)) {
#ifndef NDEBUG
            std::cerr << "failed to start apache" << std::endl;
#endif
            showError(eSTART_Apache);
            return;
        }
        connect(_apache, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(apacheFinished(int, QProcess::ExitStatus)));

#ifndef NDEBUG
#ifdef Q_OS_WIN
        std::cerr << "PID: " << _apache->pid()->dwProcessId << std::endl;
#else
        std::cerr << "PID: " << _apache->pid() << std::endl;
#endif
#endif
        _apacheStarted = true;
    }

    QString browser;
//    if(_usePortableFirefox->isChecked()) {
        browser = tr("startfirefox", "");
//    }
//    else {
////        "HKEY_CLASSES_ROOT\\HTTP\\shell\\open\\command"
////        D:\PROGRA~1\MOZILLA.ORG\SEAMON~1\SEAMON~1.EXE -url "%1"
//#ifdef Q_OS_WIN
//#define DEFBASEKEY "HTTP\\shell\\open\\command"
//
//        DWORD size = 4096;
//        char defaultBrowser[size + 1];
//        *defaultBrowser = 0;
//        HKEY reg_key;
//        LONG error = RegOpenKeyEx(HKEY_CLASSES_ROOT, DEFBASEKEY, 0, KEY_READ|KEY_QUERY_VALUE, &reg_key);
//        if(error == ERROR_SUCCESS) {
//            DWORD type = REG_SZ;
//    
//            RegQueryValueEx(reg_key, "", NULL, &type, (BYTE *)&defaultBrowser, &size);
//            RegCloseKey(reg_key);
//#ifndef NDEBUG
//            std::cerr << "browser " << defaultBrowser << std::endl;
//#endif
//        }
//        else {
//#define IEBASEKEY "Applications\\iexplore.exe\\shell\\open\\command"
//            LONG error = RegOpenKeyEx(HKEY_CLASSES_ROOT, IEBASEKEY, 0, KEY_READ|KEY_QUERY_VALUE, &reg_key);
//            if(error == ERROR_SUCCESS) {
//                DWORD type = REG_SZ;
//        
//                RegQueryValueEx(reg_key, "", NULL, &type, (BYTE *)&defaultBrowser, &size);
//                RegCloseKey(reg_key);
//#ifndef NDEBUG
//                std::cerr << "browser " << defaultBrowser << std::endl;
//#endif
//            }
//        }
//
//        browser = defaultBrowser;
//        int pos = browser.indexOf("%1");
//        if(pos > 0) {
//            browser.replace(pos, 2, tr("http://127.0.0.1:8081/test.php", ""));
//        }
//#endif
//    }

    if(!browser.isEmpty()) {
        _browser = new QProcess(this);
        _browser->start(browser);
        if(!_browser->waitForStarted(6000)) {
#ifndef NDEBUG
            std::cerr << "failed to start " << browser.toStdString() << std::endl;
#endif
            showError(eSTART_FireFox);
            return;
        }
        connect(_browser, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(ffFinished(int, QProcess::ExitStatus)));
    }

    _layers->setCurrentIndex(2);
    _timer->start();
}

QString AgWindow::getTempDir()
{
    char *temp = getenv("USERPROFILE");
    bool badChars = false;
    // do not allow german umlauts and similar, since PHP seems to have a problem
    if(temp != 0 && *temp != 0) {
        for(char const *tmp2 = temp; *tmp2; ++ tmp2) {
            if(*tmp2 < ' ' || *tmp2 >= '~') {
                badChars = true;
                break;
            }
        }
#ifndef NDEBUG
         std::cerr << "user-profile: " << temp << std::endl;
         std::cerr << "bad PHP chars: " << badChars << std::endl;
#endif
    }

    if(badChars == true)
        temp = 0;
    else {
        temp = getenv("TEMP");

        if(temp != 0 && *temp != 0 && access(temp, W_OK) != 0) {
#ifndef NDEBUG
             std::cerr << "no access 1: " << temp << std::endl;
#endif
            temp = 0;
        }
    }

    // Vista
    if(temp == 0 || *temp == 0) {
        temp = getenv("PUBLIC");
    }

    if(temp != 0 && *temp != 0 && access(temp, W_OK) != 0) {
#ifndef NDEBUG
         std::cerr << "no access 2: " << temp << std::endl;
#endif
        temp = 0;
    }

    if(temp == 0 || *temp == 0) {
        // XP
        temp = getenv("ALLUSERSPROFILE");
        if(temp != 0) {
            QDir tmpDir(temp);
            QFileInfoList content = tmpDir.entryInfoList();
            for(int i = 0; i < content.size(); ++ i) {
                if(content.at(i).baseName().isEmpty())
                    continue;
                if(content.at(i).isDir()) {
#ifndef NDEBUG
                    std::cerr << "try: " << content.at(i).absoluteFilePath().toStdString() << std::endl;
#endif
                    QString testfile = content.at(i).absoluteFilePath();
                    if(testfile.at(testfile.length() - 1) != '\\' && testfile.at(testfile.length() - 1) != '/')
#ifdef Q_OS_WIN
                        testfile += '\\';
#else
                        testfile += '/';
#endif
                    testfile += tr("mfs-wfo-apache-cb0815", "");
                    FILE *testFile = 0;
                    if((testFile = fopen(testfile.toAscii(), "w")) != 0) {
                        fwrite("test", 1, 4, testFile);
                        fclose(testFile);
                        if(access(testfile.toAscii(), W_OK) != 0) {
                            unlink(testfile.toAscii());
                        }
                        else {
                            unlink(testfile.toAscii());
                            return content.at(i).absoluteFilePath();
                        }
                    }
                }
            }
        }
    }

    if(temp != 0 && *temp != 0 && access(temp, W_OK) != 0) {
#ifndef NDEBUG
         std::cerr << "no access 3: " << temp << std::endl;
#endif
        temp = 0;
    }

    if(temp == 0 || *temp == 0) {
#ifdef Q_OS_WIN
        if(access("C:\\", W_OK) != 0) {
            showError(eSTART_EnvTemp);
            return "";
        }
        temp = "C:";
#else
        temp = "/tmp";
#endif
    }

#ifndef NDEBUG
    std::cerr << "use: " << temp << std::endl;
#endif
    return temp;
}

void AgWindow::ffFinished(int exitCode, QProcess::ExitStatus exitStatus)
{
#ifndef NDEBUG
    std::cerr << "ff finished: " << std::endl;
#endif
    showNormal();
    _layers->setCurrentIndex(0);
    _start->setText(tr("Mitgelieferten Browser neu starten", ""));
    delete _browser;
    _browser = 0;
}

void AgWindow::apacheFinished(int exitCode, QProcess::ExitStatus exitStatus)
{
    delete _apache;
    _apache = 0;
#ifdef Q_OS_WIN
	WORD wVersionRequested = MAKEWORD( 2, 2 );
	 
	WSADATA wsaData;
	int err = WSAStartup( wVersionRequested, &wsaData );
	if ( err != 0 ) {
        showError(eDIED_Apache);
        return;
	}
#endif

#define MAXRETRIES 3
    int retries;
    for(retries = 0; retries < MAXRETRIES; ++ retries) {
        struct sockaddr_in address;

        int fd;
        if((fd = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP)) == -1) {
#ifdef Q_OS_WIN
            Sleep(60);
#else
            usleep(60000);
#endif
            continue;
        }

        long addr = htonl(127*256*256*256+1);
        memset(&address, 0, sizeof(address));
        memcpy(&address.sin_addr.s_addr, &addr, 4);
        address.sin_port = htons(8081);
        address.sin_family = AF_INET;
        if(::connect(fd, (struct sockaddr *)&address, sizeof(struct sockaddr_in)) == -1) {
            ::close(fd);
#ifdef Q_OS_WIN
            if(errno == WSAEHOSTUNREACH) // No route to host
				break;
            if(errno == WSAECONNREFUSED) // Connection refused
				break;
            Sleep(30);
#else
            if(errno == EHOSTUNREACH) // No route to host
				break;
            if(errno == ECONNREFUSED) // Connection refused
				break;
            usleep(30000);
#endif
            continue;
        }

        ::close(fd);
        break;
    }

    if(retries > MAXRETRIES)
        showError(eDIED_Apache);
}

void AgWindow::cleanUp(QString const &dir, bool remove)
{
    QDir tmpDir(dir);
    QFileInfoList content = tmpDir.entryInfoList();
    for(int i = 0; i < content.size(); ++ i) {
        if(content.at(i).baseName().isEmpty())
            continue;
        if(content.at(i).isDir())
            cleanUp(content.at(i).absoluteFilePath(), true);
        else
            unlink(content.at(i).absoluteFilePath().toAscii());
    }
    if(remove) {
        int ret = rmdir(dir.toAscii());
        for(int i = 1; ret == -1 && i <= 5; ++ i) {
#ifndef NDEBUG
            std::cerr << "rmdir: " << dir.toStdString() << " = " << ret << " err: " << errno << std::endl;
#endif
#ifdef Q_OS_WIN
            Sleep(300 * i);
#else
            usleep(300000 * i);
#endif
            ret = rmdir(dir.toAscii());
        }
    }
}

void AgWindow::doAbout()
{
	AboutProgramm aboutDialog;
	aboutDialog.show();
	aboutDialog.exec();
}

#include "AgWindow.moc"
