// apachego -- a launcher for apache
//
// Copyright (C) 2007
// Wolfgang Formann
//
// this file is part of the programm patchvista.  This program is
// free software; you can redistribute it and/or modify it under the
// terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your
// option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#include <getopt.h>
#include <iostream>
#include <list>

#include <Qt/qapplication.h>
#include <Qt/qtranslator.h>
#include <Qt/qlocale.h>
#include <Qt/qicon.h>
#include <Qt/qsettings.h>

#ifdef Q_OS_WIN
#	include <io.h>
#endif

#include "ag/AgWindow.h"

static const struct option options[] = {
    { "help",          no_argument,       0, 'h'                },
    { 0,               0,                 0, 0                  }
};

static void usage(char const *progname);

int main(int argc, char **argv)
{
	QApplication app(argc, argv);
	QTranslator transApacheGo;
    if(access("apachego_de.qm", R_OK) == 0)
        transApacheGo.load("apachego_de.qm");
    else
        transApacheGo.load(":/text/apachego_de");
	app.installTranslator(&transApacheGo);

	QApplication::setWindowIcon(QIcon(":/images/prog-icon"));
	app.connect(&app, SIGNAL(lastWindowClosed()), &app, SLOT(quit()));

    int opt;

    while((opt = getopt_long(argc, argv, "h", options, 0)) != -1) {
        switch(opt) {
        case 'h':
            usage(argv[0]);
            exit(0);

		default:
            usage(argv[0]);
            exit(1);
        }
    }

	AgWindow *theWindow = AgWindow::getInstance();
	theWindow->show();
	app.exec();

    exit(0);
}

static void usage(char const *progname)
{
    std::cerr << QObject::tr("Usage", "usage").toStdString();
	    std::cerr << ": " << progname << " ";
	    std::cerr << QObject::tr("option", "usage").toStdString();
	    std::cerr << std::endl;
    std::cerr << "\t--help         -h              ";
	    std::cerr << QObject::tr("this output", "usage").toStdString();
	    std::cerr << std::endl;
}
